<?php

declare(strict_types=1);

namespace Drupal\uberflip_api;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;
use Psr\Http\Message\ResponseInterface;

/**
 * Interaction with Uberflip API.
 */
class UberflipClient implements UberflipClientInterface {

  /**
   * API base URL.
   */
  const API_BASE_URL = 'https://v2.api.uberflip.com/';

  /**
   * Constructs a new UberflipClient object.
   */
  public function __construct(
    protected Client $httpClient,
    protected LoggerChannelFactoryInterface $loggerFactory,
    protected ConfigFactory $configFactory,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function getApiToken(): string {
    // Authorize to retrieve API token.
    $response = $this->authorize();

    if ($response) {
      $data = Json::decode($response->getBody()->getContents());
      return "{$data['token_type']} {$data['access_token']}";
    }

    return '';
  }

  /**
   * Authenticate request to the Uberflip API.
   *
   * @return \Psr\Http\Message\ResponseInterface
   *   Returns false or JSON response.
   */
  public function authorize(): ?ResponseInterface {
    $body_options = [
      'grant_type' => 'client_credentials',
      'client_id' => $this->configFactory->get('uberflip_api.settings')->get('api_key'),
      'client_secret' => $this->configFactory->get('uberflip_api.settings')->get('api_secret'),
    ];
    $body_options = Json::encode($body_options);

    $options = [
      'headers' => [
        'Accept' => 'application/json',
        'Content-Type' => 'application/json',
      ],
      'body' => $body_options,
    ];

    try {
      $response = $this->httpClient->post(
        self::API_BASE_URL . 'authorize',
        $options
      );
    }
    catch (GuzzleException $e) {
      $this->loggerFactory->get('uberflip_api')
        ->error('Error @class: "%error"', [
          '@class' => get_class($e),
          '%error' => $e->getMessage(),
        ]);

      return NULL;
    }

    return $response;
  }

  /**
   * Helper function to handle the Uberflip API requests.
   *
   * @param string $method
   *   HTTP method.
   * @param string $endpoint
   *   Uberflip API endpoint.
   * @param array $options
   *   Array of request options.
   *
   * @return \Psr\Http\Message\ResponseInterface
   *   Returns false or JSON response.
   */
  public function request(string $method, string $endpoint, array $options = []): ?ResponseInterface {
    $options['headers'] = [
      'Accept' => 'application/json',
      'Content-Type' => 'application/json',
      'Authorization' => $this->getApiToken(),
    ];

    try {
      $response = $this->httpClient->{$method}(
        self::API_BASE_URL . $endpoint,
        $options
      );
    }
    catch (GuzzleException $e) {
      $this->loggerFactory->get('uberflip_api')
        ->error('Error @class: "%error"', [
          '@class' => get_class($e),
          '%error' => $e->getMessage(),
        ]);

      return NULL;
    }

    $response->getHeaders();

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function getItem(int $item_id): ?array {
    if ($response = $this->request('GET', "items/$item_id")) {
      return Json::decode($response->getBody()->getContents());
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getStreamListItems(int $stream_id): ?array {
    if ($response = $this->request('GET', "streams/$stream_id/items")) {
      $items = Json::decode($response->getBody()->getContents());

      if (!empty($items['data'])) {
        return $items['data'];
      }
    }

    return NULL;
  }

}
